@@ -15,6 +15,8 @@ load("./data/fineroot.RData")
 load("./data/insLoss.RData")
 source("./R/classMethods.R")
 source("./R/cpglm.R")
+source("./R/utilities.R")
+source("./R/bcpglm.R")
 
 #dyn.load("./src/cplm.so")
 link="log"
@@ -91,5 +93,4 @@ summary(fit3)
 
 
 
-
 }
